from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.measurement import Measurement
from ..types import UNSET, Unset

T = TypeVar("T", bound="ContainerTransferBase")


@attr.s(auto_attribs=True)
class ContainerTransferBase:
    """  """

    transfer_volume: Measurement
    source_entity_id: Union[Unset, str] = UNSET
    source_batch_id: Union[Unset, str] = UNSET
    source_container_id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        transfer_volume = self.transfer_volume.to_dict()

        source_entity_id = self.source_entity_id
        source_batch_id = self.source_batch_id
        source_container_id = self.source_container_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "transferVolume": transfer_volume,
            }
        )
        if source_entity_id is not UNSET:
            field_dict["sourceEntityId"] = source_entity_id
        if source_batch_id is not UNSET:
            field_dict["sourceBatchId"] = source_batch_id
        if source_container_id is not UNSET:
            field_dict["sourceContainerId"] = source_container_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        transfer_volume = Measurement.from_dict(d.pop("transferVolume"))

        source_entity_id = d.pop("sourceEntityId", UNSET)

        source_batch_id = d.pop("sourceBatchId", UNSET)

        source_container_id = d.pop("sourceContainerId", UNSET)

        container_transfer_base = cls(
            transfer_volume=transfer_volume,
            source_entity_id=source_entity_id,
            source_batch_id=source_batch_id,
            source_container_id=source_container_id,
        )

        return container_transfer_base



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.requests_task import RequestsTask

T = TypeVar("T", bound="RequestsTasksBulkUpdateResponse")


@attr.s(auto_attribs=True)
class RequestsTasksBulkUpdateResponse:
    """  """

    tasks: List[RequestsTask]

    def to_dict(self) -> Dict[str, Any]:
        tasks = []
        for tasks_item_data in self.tasks:
            tasks_item = tasks_item_data.to_dict()

            tasks.append(tasks_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "tasks": tasks,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        tasks = []
        _tasks = d.pop("tasks")
        for tasks_item_data in _tasks:
            tasks_item = RequestsTask.from_dict(tasks_item_data)

            tasks.append(tasks_item)

        requests_tasks_bulk_update_response = cls(
            tasks=tasks,
        )

        return requests_tasks_bulk_update_response



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.dropdown_archive_record import DropdownArchiveRecord
from ..models.dropdown_option import DropdownOption
from ..types import UNSET, Unset

T = TypeVar("T", bound="Dropdown")


@attr.s(auto_attribs=True)
class Dropdown:
    """  """

    name: str
    id: str
    options: Union[Unset, List[DropdownOption]] = UNSET
    archive_record: Union[Unset, None, DropdownArchiveRecord] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        id = self.id
        options: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.options, Unset):
            options = []
            for options_item_data in self.options:
                options_item = options_item_data.to_dict()

                options.append(options_item)

        archive_record: Union[None, Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.archive_record, Unset):
            archive_record = self.archive_record.to_dict() if self.archive_record else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "id": id,
            }
        )
        if options is not UNSET:
            field_dict["options"] = options
        if archive_record is not UNSET:
            field_dict["archiveRecord"] = archive_record

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        id = d.pop("id")

        options = []
        _options = d.pop("options", UNSET)
        for options_item_data in _options or []:
            options_item = DropdownOption.from_dict(options_item_data)

            options.append(options_item)

        archive_record = None
        _archive_record = d.pop("archiveRecord", UNSET)
        if _archive_record is not None and not isinstance(_archive_record, Unset):
            archive_record = DropdownArchiveRecord.from_dict(cast(Dict[str, Any], _archive_record))

        dropdown = cls(
            name=name,
            id=id,
            options=options,
            archive_record=archive_record,
        )

        return dropdown



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="DropdownOptionCreate")


@attr.s(auto_attribs=True)
class DropdownOptionCreate:
    """  """

    name: str

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        dropdown_option_create = cls(
            name=name,
        )

        return dropdown_option_create



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.entry_table_cell import EntryTableCell
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryTableRow")


@attr.s(auto_attribs=True)
class EntryTableRow:
    """ Each has property 'cells' that is an array of cell objects """

    cells: Union[Unset, List[EntryTableCell]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        cells: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.cells, Unset):
            cells = []
            for cells_item_data in self.cells:
                cells_item = cells_item_data.to_dict()

                cells.append(cells_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if cells is not UNSET:
            field_dict["cells"] = cells

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        cells = []
        _cells = d.pop("cells", UNSET)
        for cells_item_data in _cells or []:
            cells_item = EntryTableCell.from_dict(cells_item_data)

            cells.append(cells_item)

        entry_table_row = cls(
            cells=cells,
        )

        return entry_table_row



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.assay_result import AssayResult
from ..models.request_response_samples_item import RequestResponseSamplesItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestResponse")


@attr.s(auto_attribs=True)
class RequestResponse:
    """  """

    samples: Union[Unset, List[RequestResponseSamplesItem]] = UNSET
    results: Union[Unset, List[AssayResult]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        samples: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.samples, Unset):
            samples = []
            for samples_item_data in self.samples:
                samples_item = samples_item_data.to_dict()

                samples.append(samples_item)

        results: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.results, Unset):
            results = []
            for results_item_data in self.results:
                results_item = results_item_data.to_dict()

                results.append(results_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if samples is not UNSET:
            field_dict["samples"] = samples
        if results is not UNSET:
            field_dict["results"] = results

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        samples = []
        _samples = d.pop("samples", UNSET)
        for samples_item_data in _samples or []:
            samples_item = RequestResponseSamplesItem.from_dict(samples_item_data)

            samples.append(samples_item)

        results = []
        _results = d.pop("results", UNSET)
        for results_item_data in _results or []:
            results_item = AssayResult.from_dict(results_item_data)

            results.append(results_item)

        request_response = cls(
            samples=samples,
            results=results,
        )

        return request_response



from typing import Any, Dict, Type, TypeVar, Union, cast

import attr

from ..models.automation_file_automation_file_config import AutomationFileAutomationFileConfig
from ..models.automation_file_file import AutomationFileFile
from ..types import UNSET, Unset

T = TypeVar("T", bound="AutomationInputGenerator")


@attr.s(auto_attribs=True)
class AutomationInputGenerator:
    """  """

    api_url: Union[Unset, str] = UNSET
    assay_run_id: Union[Unset, str] = UNSET
    automation_file_config: Union[Unset, AutomationFileAutomationFileConfig] = UNSET
    file: Union[Unset, AutomationFileFile] = UNSET
    id: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        api_url = self.api_url
        assay_run_id = self.assay_run_id
        automation_file_config: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.automation_file_config, Unset):
            automation_file_config = self.automation_file_config.to_dict()

        file: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.file, Unset):
            file = self.file.to_dict()

        id = self.id
        status = self.status

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if api_url is not UNSET:
            field_dict["apiURL"] = api_url
        if assay_run_id is not UNSET:
            field_dict["assayRunId"] = assay_run_id
        if automation_file_config is not UNSET:
            field_dict["automationFileConfig"] = automation_file_config
        if file is not UNSET:
            field_dict["file"] = file
        if id is not UNSET:
            field_dict["id"] = id
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        api_url = d.pop("apiURL", UNSET)

        assay_run_id = d.pop("assayRunId", UNSET)

        automation_file_config: Union[Unset, AutomationFileAutomationFileConfig] = UNSET
        _automation_file_config = d.pop("automationFileConfig", UNSET)
        if _automation_file_config is not None and not isinstance(_automation_file_config, Unset):
            automation_file_config = AutomationFileAutomationFileConfig.from_dict(
                cast(Dict[str, Any], _automation_file_config)
            )

        file: Union[Unset, AutomationFileFile] = UNSET
        _file = d.pop("file", UNSET)
        if _file is not None and not isinstance(_file, Unset):
            file = AutomationFileFile.from_dict(cast(Dict[str, Any], _file))

        id = d.pop("id", UNSET)

        status = d.pop("status", UNSET)

        automation_input_generator = cls(
            api_url=api_url,
            assay_run_id=assay_run_id,
            automation_file_config=automation_file_config,
            file=file,
            id=id,
            status=status,
        )

        return automation_input_generator



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="AsyncTaskLink")


@attr.s(auto_attribs=True)
class AsyncTaskLink:
    """  """

    task_id: str

    def to_dict(self) -> Dict[str, Any]:
        task_id = self.task_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "taskId": task_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        task_id = d.pop("taskId")

        async_task_link = cls(
            task_id=task_id,
        )

        return async_task_link



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.entry_link import EntryLink
from ..models.entry_table import EntryTable
from ..models.note_part_type import NotePartType
from ..types import UNSET, Unset

T = TypeVar("T", bound="NotePart")


@attr.s(auto_attribs=True)
class NotePart:
    """Notes are the main building blocks of entries. Each note corresponds roughly to a paragraph and has one of these types: - 'text': plain text - 'code': preformatted code block - 'table': a table with rows and columns of text - 'list_bullet': one "line" of a bulleted list - 'list_number': one "line" of a numbered list - 'list_checkbox': one "line" of a checklist - 'external_file': an attached user-uploaded file"""

    type: Union[Unset, NotePartType] = UNSET
    indentation: Union[Unset, int] = 0
    text: Union[Unset, str] = UNSET
    links: Union[Unset, List[EntryLink]] = UNSET
    checked: Union[Unset, bool] = UNSET
    table: Union[Unset, EntryTable] = UNSET
    external_file_id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        type: Union[Unset, NotePartType] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        indentation = self.indentation
        text = self.text
        links: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.links, Unset):
            links = []
            for links_item_data in self.links:
                links_item = links_item_data.to_dict()

                links.append(links_item)

        checked = self.checked
        table: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.table, Unset):
            table = self.table.to_dict()

        external_file_id = self.external_file_id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if indentation is not UNSET:
            field_dict["indentation"] = indentation
        if text is not UNSET:
            field_dict["text"] = text
        if links is not UNSET:
            field_dict["links"] = links
        if checked is not UNSET:
            field_dict["checked"] = checked
        if table is not UNSET:
            field_dict["table"] = table
        if external_file_id is not UNSET:
            field_dict["externalFileId"] = external_file_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        type = None
        _type = d.pop("type", UNSET)
        if _type is not None and _type is not UNSET:
            type = NotePartType(_type)

        indentation = d.pop("indentation", UNSET)

        text = d.pop("text", UNSET)

        links = []
        _links = d.pop("links", UNSET)
        for links_item_data in _links or []:
            links_item = EntryLink.from_dict(links_item_data)

            links.append(links_item)

        checked = d.pop("checked", UNSET)

        table: Union[Unset, EntryTable] = UNSET
        _table = d.pop("table", UNSET)
        if _table is not None and not isinstance(_table, Unset):
            table = EntryTable.from_dict(cast(Dict[str, Any], _table))

        external_file_id = d.pop("externalFileId", UNSET)

        note_part = cls(
            type=type,
            indentation=indentation,
            text=text,
            links=links,
            checked=checked,
            table=table,
            external_file_id=external_file_id,
        )

        return note_part



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="BlobPart")


@attr.s(auto_attribs=True)
class BlobPart:
    """  """

    part_number: Union[Unset, int] = UNSET
    e_tag: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        part_number = self.part_number
        e_tag = self.e_tag

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if part_number is not UNSET:
            field_dict["partNumber"] = part_number
        if e_tag is not UNSET:
            field_dict["eTag"] = e_tag

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        part_number = d.pop("partNumber", UNSET)

        e_tag = d.pop("eTag", UNSET)

        blob_part = cls(
            part_number=part_number,
            e_tag=e_tag,
        )

        return blob_part



from typing import Any, Dict, Type, TypeVar, Union, cast

import attr

from ..models.async_task_errors import AsyncTaskErrors
from ..models.async_task_response import AsyncTaskResponse
from ..models.async_task_status import AsyncTaskStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="AsyncTask")


@attr.s(auto_attribs=True)
class AsyncTask:
    """  """

    status: AsyncTaskStatus
    response: Union[Unset, AsyncTaskResponse] = UNSET
    message: Union[Unset, str] = UNSET
    errors: Union[Unset, AsyncTaskErrors] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        status = self.status.value

        response: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.response, Unset):
            response = self.response.to_dict()

        message = self.message
        errors: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.errors, Unset):
            errors = self.errors.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "status": status,
            }
        )
        if response is not UNSET:
            field_dict["response"] = response
        if message is not UNSET:
            field_dict["message"] = message
        if errors is not UNSET:
            field_dict["errors"] = errors

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = AsyncTaskStatus(d.pop("status"))

        response: Union[Unset, AsyncTaskResponse] = UNSET
        _response = d.pop("response", UNSET)
        if _response is not None and not isinstance(_response, Unset):
            response = AsyncTaskResponse.from_dict(cast(Dict[str, Any], _response))

        message = d.pop("message", UNSET)

        errors: Union[Unset, AsyncTaskErrors] = UNSET
        _errors = d.pop("errors", UNSET)
        if _errors is not None and not isinstance(_errors, Unset):
            errors = AsyncTaskErrors.from_dict(cast(Dict[str, Any], _errors))

        async_task = cls(
            status=status,
            response=response,
            message=message,
            errors=errors,
        )

        return async_task



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

T = TypeVar("T", bound="UnregisterEntities")


@attr.s(auto_attribs=True)
class UnregisterEntities:
    """  """

    entity_ids: List[str]
    folder_id: str

    def to_dict(self) -> Dict[str, Any]:
        entity_ids = self.entity_ids

        folder_id = self.folder_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "entityIds": entity_ids,
                "folderId": folder_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        entity_ids = cast(List[str], d.pop("entityIds"))

        folder_id = d.pop("folderId")

        unregister_entities = cls(
            entity_ids=entity_ids,
            folder_id=folder_id,
        )

        return unregister_entities



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.schema_field import SchemaField
from ..types import UNSET, Unset

T = TypeVar("T", bound="Schema")


@attr.s(auto_attribs=True)
class Schema:
    """  """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    field_definitions: Union[Unset, List[SchemaField]] = UNSET
    type: Union[Unset, str] = UNSET
    prefix: Union[Unset, str] = UNSET
    registry_id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        field_definitions: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.field_definitions, Unset):
            field_definitions = []
            for field_definitions_item_data in self.field_definitions:
                field_definitions_item = field_definitions_item_data.to_dict()

                field_definitions.append(field_definitions_item)

        type = self.type
        prefix = self.prefix
        registry_id = self.registry_id

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if field_definitions is not UNSET:
            field_dict["fieldDefinitions"] = field_definitions
        if type is not UNSET:
            field_dict["type"] = type
        if prefix is not UNSET:
            field_dict["prefix"] = prefix
        if registry_id is not UNSET:
            field_dict["registryId"] = registry_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        field_definitions = []
        _field_definitions = d.pop("fieldDefinitions", UNSET)
        for field_definitions_item_data in _field_definitions or []:
            field_definitions_item = SchemaField.from_dict(field_definitions_item_data)

            field_definitions.append(field_definitions_item)

        type = d.pop("type", UNSET)

        prefix = d.pop("prefix", UNSET)

        registry_id = d.pop("registryId", UNSET)

        schema = cls(
            id=id,
            name=name,
            field_definitions=field_definitions,
            type=type,
            prefix=prefix,
            registry_id=registry_id,
        )

        return schema



from typing import Any, Dict, Type, TypeVar, Union, cast

import attr

from ..models.fields import Fields
from ..types import UNSET, Unset

T = TypeVar("T", bound="AssayRunUpdate")


@attr.s(auto_attribs=True)
class AssayRunUpdate:
    """  """

    fields: Union[Unset, Fields] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.fields, Unset):
            fields = self.fields.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if fields is not UNSET:
            field_dict["fields"] = fields

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        fields: Union[Unset, Fields] = UNSET
        _fields = d.pop("fields", UNSET)
        if _fields is not None and not isinstance(_fields, Unset):
            fields = Fields.from_dict(cast(Dict[str, Any], _fields))

        assay_run_update = cls(
            fields=fields,
        )

        return assay_run_update



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.event import Event

T = TypeVar("T", bound="EventsPaginatedList")


@attr.s(auto_attribs=True)
class EventsPaginatedList:
    """  """

    events: List[Event]
    next_token: str

    def to_dict(self) -> Dict[str, Any]:
        events = []
        for events_item_data in self.events:
            events_item = events_item_data.to_dict()

            events.append(events_item)

        next_token = self.next_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "events": events,
                "nextToken": next_token,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        events = []
        _events = d.pop("events")
        for events_item_data in _events:
            events_item = Event.from_dict(events_item_data)

            events.append(events_item)

        next_token = d.pop("nextToken")

        events_paginated_list = cls(
            events=events,
            next_token=next_token,
        )

        return events_paginated_list



from typing import Any, Dict, Type, TypeVar, Union, cast

import attr

from ..models.entry_link import EntryLink
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryTableCell")


@attr.s(auto_attribs=True)
class EntryTableCell:
    """  """

    text: Union[Unset, str] = UNSET
    link: Union[Unset, EntryLink] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        text = self.text
        link: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.link, Unset):
            link = self.link.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if text is not UNSET:
            field_dict["text"] = text
        if link is not UNSET:
            field_dict["link"] = link

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        text = d.pop("text", UNSET)

        link: Union[Unset, EntryLink] = UNSET
        _link = d.pop("link", UNSET)
        if _link is not None and not isinstance(_link, Unset):
            link = EntryLink.from_dict(cast(Dict[str, Any], _link))

        entry_table_cell = cls(
            text=text,
            link=link,
        )

        return entry_table_cell



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.assay_result_schema_archive_record import AssayResultSchemaArchiveRecord
from ..models.assay_result_schema_organization import AssayResultSchemaOrganization
from ..models.assay_result_schema_type import AssayResultSchemaType
from ..models.schema_field import SchemaField
from ..types import UNSET, Unset

T = TypeVar("T", bound="AssayResultSchema")


@attr.s(auto_attribs=True)
class AssayResultSchema:
    """  """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    archive_record: Union[Unset, None, AssayResultSchemaArchiveRecord] = UNSET
    field_definitions: Union[Unset, List[SchemaField]] = UNSET
    type: Union[Unset, AssayResultSchemaType] = UNSET
    system_name: Union[Unset, str] = UNSET
    derived_from: Union[Unset, None, str] = UNSET
    organization: Union[Unset, AssayResultSchemaOrganization] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        archive_record: Union[None, Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.archive_record, Unset):
            archive_record = self.archive_record.to_dict() if self.archive_record else None

        field_definitions: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.field_definitions, Unset):
            field_definitions = []
            for field_definitions_item_data in self.field_definitions:
                field_definitions_item = field_definitions_item_data.to_dict()

                field_definitions.append(field_definitions_item)

        type: Union[Unset, AssayResultSchemaType] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        system_name = self.system_name
        derived_from = self.derived_from
        organization: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.organization, Unset):
            organization = self.organization.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if archive_record is not UNSET:
            field_dict["archiveRecord"] = archive_record
        if field_definitions is not UNSET:
            field_dict["fieldDefinitions"] = field_definitions
        if type is not UNSET:
            field_dict["type"] = type
        if system_name is not UNSET:
            field_dict["systemName"] = system_name
        if derived_from is not UNSET:
            field_dict["derivedFrom"] = derived_from
        if organization is not UNSET:
            field_dict["organization"] = organization

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        archive_record = None
        _archive_record = d.pop("archiveRecord", UNSET)
        if _archive_record is not None and not isinstance(_archive_record, Unset):
            archive_record = AssayResultSchemaArchiveRecord.from_dict(cast(Dict[str, Any], _archive_record))

        field_definitions = []
        _field_definitions = d.pop("fieldDefinitions", UNSET)
        for field_definitions_item_data in _field_definitions or []:
            field_definitions_item = SchemaField.from_dict(field_definitions_item_data)

            field_definitions.append(field_definitions_item)

        type = None
        _type = d.pop("type", UNSET)
        if _type is not None and _type is not UNSET:
            type = AssayResultSchemaType(_type)

        system_name = d.pop("systemName", UNSET)

        derived_from = d.pop("derivedFrom", UNSET)

        organization: Union[Unset, AssayResultSchemaOrganization] = UNSET
        _organization = d.pop("organization", UNSET)
        if _organization is not None and not isinstance(_organization, Unset):
            organization = AssayResultSchemaOrganization.from_dict(cast(Dict[str, Any], _organization))

        assay_result_schema = cls(
            id=id,
            name=name,
            archive_record=archive_record,
            field_definitions=field_definitions,
            type=type,
            system_name=system_name,
            derived_from=derived_from,
            organization=organization,
        )

        return assay_result_schema



from enum import Enum


class NamingStrategy(str, Enum):
    NEW_IDS = "NEW_IDS"
    IDS_FROM_NAMES = "IDS_FROM_NAMES"
    DELETE_NAMES = "DELETE_NAMES"
    SET_FROM_NAME_PARTS = "SET_FROM_NAME_PARTS"
    REPLACE_NAMES_FROM_PARTS = "REPLACE_NAMES_FROM_PARTS"
    KEEP_NAMES = "KEEP_NAMES"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class OligosArchiveReason(str, Enum):
    MADE_IN_ERROR = "Made in error"
    RETIRED = "Retired"
    EXPENDED = "Expended"
    SHIPPED = "Shipped"
    CONTAMINATED = "Contaminated"
    EXPIRED = "Expired"
    MISSING = "Missing"
    OTHER = "Other"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

T = TypeVar("T", bound="LocationsUnarchive")


@attr.s(auto_attribs=True)
class LocationsUnarchive:
    """  """

    location_ids: List[str]

    def to_dict(self) -> Dict[str, Any]:
        location_ids = self.location_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "locationIds": location_ids,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        location_ids = cast(List[str], d.pop("locationIds"))

        locations_unarchive = cls(
            location_ids=location_ids,
        )

        return locations_unarchive



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.locations_archive_reason import LocationsArchiveReason
from ..types import UNSET, Unset

T = TypeVar("T", bound="LocationsArchive")


@attr.s(auto_attribs=True)
class LocationsArchive:
    """  """

    location_ids: List[str]
    reason: LocationsArchiveReason
    should_remove_barcodes: Union[Unset, bool] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        location_ids = self.location_ids

        reason = self.reason.value

        should_remove_barcodes = self.should_remove_barcodes

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "locationIds": location_ids,
                "reason": reason,
            }
        )
        if should_remove_barcodes is not UNSET:
            field_dict["shouldRemoveBarcodes"] = should_remove_barcodes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        location_ids = cast(List[str], d.pop("locationIds"))

        reason = LocationsArchiveReason(d.pop("reason"))

        should_remove_barcodes = d.pop("shouldRemoveBarcodes", UNSET)

        locations_archive = cls(
            location_ids=location_ids,
            reason=reason,
            should_remove_barcodes=should_remove_barcodes,
        )

        return locations_archive



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.barcode_validation_result import BarcodeValidationResult
from ..types import UNSET, Unset

T = TypeVar("T", bound="BarcodeValidationResults")


@attr.s(auto_attribs=True)
class BarcodeValidationResults:
    """  """

    validation_results: Union[Unset, List[BarcodeValidationResult]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        validation_results: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.validation_results, Unset):
            validation_results = []
            for validation_results_item_data in self.validation_results:
                validation_results_item = validation_results_item_data.to_dict()

                validation_results.append(validation_results_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if validation_results is not UNSET:
            field_dict["validationResults"] = validation_results

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        validation_results = []
        _validation_results = d.pop("validationResults", UNSET)
        for validation_results_item_data in _validation_results or []:
            validation_results_item = BarcodeValidationResult.from_dict(validation_results_item_data)

            validation_results.append(validation_results_item)

        barcode_validation_results = cls(
            validation_results=validation_results,
        )

        return barcode_validation_results



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CustomEntitiesArchivalChange")


@attr.s(auto_attribs=True)
class CustomEntitiesArchivalChange:
    """IDs of all items that were archived or unarchived, grouped by resource type. This includes the IDs of custom entities along with any IDs of batches that were archived (or unarchived)."""

    custom_entity_ids: Union[Unset, List[str]] = UNSET
    batch_ids: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        custom_entity_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.custom_entity_ids, Unset):
            custom_entity_ids = self.custom_entity_ids

        batch_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.batch_ids, Unset):
            batch_ids = self.batch_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if custom_entity_ids is not UNSET:
            field_dict["customEntityIds"] = custom_entity_ids
        if batch_ids is not UNSET:
            field_dict["batchIds"] = batch_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        custom_entity_ids = cast(List[str], d.pop("customEntityIds", UNSET))

        batch_ids = cast(List[str], d.pop("batchIds", UNSET))

        custom_entities_archival_change = cls(
            custom_entity_ids=custom_entity_ids,
            batch_ids=batch_ids,
        )

        return custom_entities_archival_change



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.oligo_bulk_create import OligoBulkCreate
from ..types import UNSET, Unset

T = TypeVar("T", bound="OligosBulkCreateRequest")


@attr.s(auto_attribs=True)
class OligosBulkCreateRequest:
    """  """

    oligos: Union[Unset, List[OligoBulkCreate]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        oligos: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.oligos, Unset):
            oligos = []
            for oligos_item_data in self.oligos:
                oligos_item = oligos_item_data.to_dict()

                oligos.append(oligos_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if oligos is not UNSET:
            field_dict["oligos"] = oligos

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        oligos = []
        _oligos = d.pop("oligos", UNSET)
        for oligos_item_data in _oligos or []:
            oligos_item = OligoBulkCreate.from_dict(oligos_item_data)

            oligos.append(oligos_item)

        oligos_bulk_create_request = cls(
            oligos=oligos,
        )

        return oligos_bulk_create_request



import datetime
from typing import Any, Dict, List, Optional, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.archive_record import ArchiveRecord
from ..models.checkout_record import CheckoutRecord
from ..models.container_content import ContainerContent
from ..models.fields import Fields
from ..models.measurement import Measurement
from ..models.schema_summary import SchemaSummary
from ..models.user_summary import UserSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="Container")


@attr.s(auto_attribs=True)
class Container:
    """  """

    id: str
    name: str
    barcode: str
    checkout_record: CheckoutRecord
    contents: List[ContainerContent]
    created_at: datetime.datetime
    creator: UserSummary
    fields: Fields
    modified_at: datetime.datetime
    parent_storage_id: str
    parent_storage_schema: SchemaSummary
    schema: SchemaSummary
    volume: Measurement
    web_url: str
    project_id: Optional[str]
    archive_record: Union[Unset, ArchiveRecord] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        barcode = self.barcode
        checkout_record = self.checkout_record.to_dict()

        contents = []
        for contents_item_data in self.contents:
            contents_item = contents_item_data.to_dict()

            contents.append(contents_item)

        created_at = self.created_at.isoformat()

        creator = self.creator.to_dict()

        fields = self.fields.to_dict()

        modified_at = self.modified_at.isoformat()

        parent_storage_id = self.parent_storage_id
        parent_storage_schema = self.parent_storage_schema.to_dict()

        schema = self.schema.to_dict()

        volume = self.volume.to_dict()

        web_url = self.web_url
        archive_record: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.archive_record, Unset):
            archive_record = self.archive_record.to_dict()

        project_id = self.project_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "id": id,
                "name": name,
                "barcode": barcode,
                "checkoutRecord": checkout_record,
                "contents": contents,
                "createdAt": created_at,
                "creator": creator,
                "fields": fields,
                "modifiedAt": modified_at,
                "parentStorageId": parent_storage_id,
                "parentStorageSchema": parent_storage_schema,
                "schema": schema,
                "volume": volume,
                "webURL": web_url,
                "projectId": project_id,
            }
        )
        if archive_record is not UNSET:
            field_dict["archiveRecord"] = archive_record

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        name = d.pop("name")

        barcode = d.pop("barcode")

        checkout_record = CheckoutRecord.from_dict(d.pop("checkoutRecord"))

        contents = []
        _contents = d.pop("contents")
        for contents_item_data in _contents:
            contents_item = ContainerContent.from_dict(contents_item_data)

            contents.append(contents_item)

        created_at = isoparse(d.pop("createdAt"))

        creator = UserSummary.from_dict(d.pop("creator"))

        fields = Fields.from_dict(d.pop("fields"))

        modified_at = isoparse(d.pop("modifiedAt"))

        parent_storage_id = d.pop("parentStorageId")

        parent_storage_schema = SchemaSummary.from_dict(d.pop("parentStorageSchema"))

        schema = SchemaSummary.from_dict(d.pop("schema"))

        volume = Measurement.from_dict(d.pop("volume"))

        web_url = d.pop("webURL")

        archive_record: Union[Unset, ArchiveRecord] = UNSET
        _archive_record = d.pop("archiveRecord", UNSET)
        if _archive_record is not None and not isinstance(_archive_record, Unset):
            archive_record = ArchiveRecord.from_dict(cast(Dict[str, Any], _archive_record))

        project_id = d.pop("projectId")

        container = cls(
            id=id,
            name=name,
            barcode=barcode,
            checkout_record=checkout_record,
            contents=contents,
            created_at=created_at,
            creator=creator,
            fields=fields,
            modified_at=modified_at,
            parent_storage_id=parent_storage_id,
            parent_storage_schema=parent_storage_schema,
            schema=schema,
            volume=volume,
            web_url=web_url,
            archive_record=archive_record,
            project_id=project_id,
        )

        return container



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.blob_type import BlobType
from ..models.blob_upload_status import BlobUploadStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="Blob")


@attr.s(auto_attribs=True)
class Blob:
    """  """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    type: Union[Unset, BlobType] = UNSET
    mime_type: Union[Unset, str] = UNSET
    upload_status: Union[Unset, BlobUploadStatus] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        type: Union[Unset, BlobType] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        mime_type = self.mime_type
        upload_status: Union[Unset, BlobUploadStatus] = UNSET
        if not isinstance(self.upload_status, Unset):
            upload_status = self.upload_status

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if type is not UNSET:
            field_dict["type"] = type
        if mime_type is not UNSET:
            field_dict["mimeType"] = mime_type
        if upload_status is not UNSET:
            field_dict["uploadStatus"] = upload_status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        type = None
        _type = d.pop("type", UNSET)
        if _type is not None and _type is not UNSET:
            type = BlobType(_type)

        mime_type = d.pop("mimeType", UNSET)

        upload_status = None
        _upload_status = d.pop("uploadStatus", UNSET)
        if _upload_status is not None and _upload_status is not UNSET:
            upload_status = BlobUploadStatus(_upload_status)

        blob = cls(
            id=id,
            name=name,
            type=type,
            mime_type=mime_type,
            upload_status=upload_status,
        )

        return blob



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.translation_regions_item import TranslationRegionsItem
from ..types import UNSET, Unset

T = TypeVar("T", bound="Translation")


@attr.s(auto_attribs=True)
class Translation:
    """  """

    start: Union[Unset, int] = UNSET
    end: Union[Unset, int] = UNSET
    strand: Union[Unset, int] = UNSET
    amino_acids: Union[Unset, str] = UNSET
    regions: Union[Unset, List[TranslationRegionsItem]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        start = self.start
        end = self.end
        strand = self.strand
        amino_acids = self.amino_acids
        regions: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.regions, Unset):
            regions = []
            for regions_item_data in self.regions:
                regions_item = regions_item_data.to_dict()

                regions.append(regions_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end
        if strand is not UNSET:
            field_dict["strand"] = strand
        if amino_acids is not UNSET:
            field_dict["aminoAcids"] = amino_acids
        if regions is not UNSET:
            field_dict["regions"] = regions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        start = d.pop("start", UNSET)

        end = d.pop("end", UNSET)

        strand = d.pop("strand", UNSET)

        amino_acids = d.pop("aminoAcids", UNSET)

        regions = []
        _regions = d.pop("regions", UNSET)
        for regions_item_data in _regions or []:
            regions_item = TranslationRegionsItem.from_dict(regions_item_data)

            regions.append(regions_item)

        translation = cls(
            start=start,
            end=end,
            strand=strand,
            amino_acids=amino_acids,
            regions=regions,
        )

        return translation



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.plate import Plate

T = TypeVar("T", bound="PlatesPaginatedList")


@attr.s(auto_attribs=True)
class PlatesPaginatedList:
    """  """

    next_token: str
    plates: List[Plate]

    def to_dict(self) -> Dict[str, Any]:
        next_token = self.next_token
        plates = []
        for plates_item_data in self.plates:
            plates_item = plates_item_data.to_dict()

            plates.append(plates_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "nextToken": next_token,
                "plates": plates,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        next_token = d.pop("nextToken")

        plates = []
        _plates = d.pop("plates")
        for plates_item_data in _plates:
            plates_item = Plate.from_dict(plates_item_data)

            plates.append(plates_item)

        plates_paginated_list = cls(
            next_token=next_token,
            plates=plates,
        )

        return plates_paginated_list



from typing import Any, Dict, List, Optional, Type, TypeVar, Union, cast

import attr

from ..models.entry_table_row import EntryTableRow
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryTable")


@attr.s(auto_attribs=True)
class EntryTable:
    """Actual tabular data with rows and columns of text on the note."""

    name: Union[Unset, str] = UNSET
    column_labels: Union[Unset, List[Optional[str]]] = UNSET
    rows: Union[Unset, List[EntryTableRow]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        column_labels: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.column_labels, Unset):
            column_labels = self.column_labels

        rows: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.rows, Unset):
            rows = []
            for rows_item_data in self.rows:
                rows_item = rows_item_data.to_dict()

                rows.append(rows_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if column_labels is not UNSET:
            field_dict["columnLabels"] = column_labels
        if rows is not UNSET:
            field_dict["rows"] = rows

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET)

        column_labels = cast(List[Optional[str]], d.pop("columnLabels", UNSET))

        rows = []
        _rows = d.pop("rows", UNSET)
        for rows_item_data in _rows or []:
            rows_item = EntryTableRow.from_dict(rows_item_data)

            rows.append(rows_item)

        entry_table = cls(
            name=name,
            column_labels=column_labels,
            rows=rows,
        )

        return entry_table



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.request_schema import RequestSchema

T = TypeVar("T", bound="RequestSchemasPaginatedList")


@attr.s(auto_attribs=True)
class RequestSchemasPaginatedList:
    """  """

    request_schemas: List[RequestSchema]
    next_token: str

    def to_dict(self) -> Dict[str, Any]:
        request_schemas = []
        for request_schemas_item_data in self.request_schemas:
            request_schemas_item = request_schemas_item_data.to_dict()

            request_schemas.append(request_schemas_item)

        next_token = self.next_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "requestSchemas": request_schemas,
                "nextToken": next_token,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        request_schemas = []
        _request_schemas = d.pop("requestSchemas")
        for request_schemas_item_data in _request_schemas:
            request_schemas_item = RequestSchema.from_dict(request_schemas_item_data)

            request_schemas.append(request_schemas_item)

        next_token = d.pop("nextToken")

        request_schemas_paginated_list = cls(
            request_schemas=request_schemas,
            next_token=next_token,
        )

        return request_schemas_paginated_list



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.note_part import NotePart
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryDay")


@attr.s(auto_attribs=True)
class EntryDay:
    """  """

    date: Union[Unset, str] = UNSET
    notes: Union[Unset, List[NotePart]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        date = self.date
        notes: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.notes, Unset):
            notes = []
            for notes_item_data in self.notes:
                notes_item = notes_item_data.to_dict()

                notes.append(notes_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if date is not UNSET:
            field_dict["date"] = date
        if notes is not UNSET:
            field_dict["notes"] = notes

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        date = d.pop("date", UNSET)

        notes = []
        _notes = d.pop("notes", UNSET)
        for notes_item_data in _notes or []:
            notes_item = NotePart.from_dict(notes_item_data)

            notes.append(notes_item)

        entry_day = cls(
            date=date,
            notes=notes,
        )

        return entry_day



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.team_summary import TeamSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestAssigneesItem")


@attr.s(auto_attribs=True)
class RequestAssigneesItem:
    """  """

    team: Union[Unset, TeamSummary] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        team: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.team, Unset):
            team = self.team.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if team is not UNSET:
            field_dict["team"] = team

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        team: Union[Unset, TeamSummary] = UNSET
        _team = d.pop("team", UNSET)
        if _team is not None and not isinstance(_team, Unset):
            team = TeamSummary.from_dict(cast(Dict[str, Any], _team))

        request_assignees_item = cls(
            team=team,
        )

        request_assignees_item.additional_properties = d
        return request_assignees_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="BlobPartCreate")


@attr.s(auto_attribs=True)
class BlobPartCreate:
    """  """

    part_number: int
    data64: str
    md5: str

    def to_dict(self) -> Dict[str, Any]:
        part_number = self.part_number
        data64 = self.data64
        md5 = self.md5

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "partNumber": part_number,
                "data64": data64,
                "md5": md5,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        part_number = d.pop("partNumber")

        data64 = d.pop("data64")

        md5 = d.pop("md5")

        blob_part_create = cls(
            part_number=part_number,
            data64=data64,
            md5=md5,
        )

        return blob_part_create



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.blob_part import BlobPart
from ..types import UNSET, Unset

T = TypeVar("T", bound="BlobComplete")


@attr.s(auto_attribs=True)
class BlobComplete:
    """  """

    parts: Union[Unset, List[BlobPart]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        parts: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.parts, Unset):
            parts = []
            for parts_item_data in self.parts:
                parts_item = parts_item_data.to_dict()

                parts.append(parts_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if parts is not UNSET:
            field_dict["parts"] = parts

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        parts = []
        _parts = d.pop("parts", UNSET)
        for parts_item_data in _parts or []:
            parts_item = BlobPart.from_dict(parts_item_data)

            parts.append(parts_item)

        blob_complete = cls(
            parts=parts,
        )

        return blob_complete



from enum import Enum


class UserValidationValidationStatus(str, Enum):
    VALID = "VALID"
    INVALID = "INVALID"
    PARTIALLY_VALID = "PARTIALLY_VALID"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="Pagination")


@attr.s(auto_attribs=True)
class Pagination:
    """  """

    next_token: str

    def to_dict(self) -> Dict[str, Any]:
        next_token = self.next_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "nextToken": next_token,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        next_token = d.pop("nextToken")

        pagination = cls(
            next_token=next_token,
        )

        return pagination



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.container_transfer_destination_contents_item import ContainerTransferDestinationContentsItem
from ..models.measurement import Measurement
from ..types import UNSET, Unset

T = TypeVar("T", bound="ContainerTransfer")


@attr.s(auto_attribs=True)
class ContainerTransfer:
    """  """

    transfer_volume: Measurement
    destination_volume: Union[Unset, Measurement] = UNSET
    destination_contents: Union[Unset, List[ContainerTransferDestinationContentsItem]] = UNSET
    source_entity_id: Union[Unset, str] = UNSET
    source_batch_id: Union[Unset, str] = UNSET
    source_container_id: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        transfer_volume = self.transfer_volume.to_dict()

        destination_volume: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.destination_volume, Unset):
            destination_volume = self.destination_volume.to_dict()

        destination_contents: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.destination_contents, Unset):
            destination_contents = []
            for destination_contents_item_data in self.destination_contents:
                destination_contents_item = destination_contents_item_data.to_dict()

                destination_contents.append(destination_contents_item)

        source_entity_id = self.source_entity_id
        source_batch_id = self.source_batch_id
        source_container_id = self.source_container_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "transferVolume": transfer_volume,
            }
        )
        if destination_volume is not UNSET:
            field_dict["destinationVolume"] = destination_volume
        if destination_contents is not UNSET:
            field_dict["destinationContents"] = destination_contents
        if source_entity_id is not UNSET:
            field_dict["sourceEntityId"] = source_entity_id
        if source_batch_id is not UNSET:
            field_dict["sourceBatchId"] = source_batch_id
        if source_container_id is not UNSET:
            field_dict["sourceContainerId"] = source_container_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        transfer_volume = Measurement.from_dict(d.pop("transferVolume"))

        destination_volume: Union[Unset, Measurement] = UNSET
        _destination_volume = d.pop("destinationVolume", UNSET)
        if _destination_volume is not None and not isinstance(_destination_volume, Unset):
            destination_volume = Measurement.from_dict(cast(Dict[str, Any], _destination_volume))

        destination_contents = []
        _destination_contents = d.pop("destinationContents", UNSET)
        for destination_contents_item_data in _destination_contents or []:
            destination_contents_item = ContainerTransferDestinationContentsItem.from_dict(
                destination_contents_item_data
            )

            destination_contents.append(destination_contents_item)

        source_entity_id = d.pop("sourceEntityId", UNSET)

        source_batch_id = d.pop("sourceBatchId", UNSET)

        source_container_id = d.pop("sourceContainerId", UNSET)

        container_transfer = cls(
            transfer_volume=transfer_volume,
            destination_volume=destination_volume,
            destination_contents=destination_contents,
            source_entity_id=source_entity_id,
            source_batch_id=source_batch_id,
            source_container_id=source_container_id,
        )

        return container_transfer



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="TagSchemaConstraint")


@attr.s(auto_attribs=True)
class TagSchemaConstraint:
    """  """

    field_definition_names: Union[Unset, List[str]] = UNSET
    has_unique_residues: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        field_definition_names: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.field_definition_names, Unset):
            field_definition_names = self.field_definition_names

        has_unique_residues = self.has_unique_residues

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if field_definition_names is not UNSET:
            field_dict["fieldDefinitionNames"] = field_definition_names
        if has_unique_residues is not UNSET:
            field_dict["hasUniqueResidues"] = has_unique_residues

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        field_definition_names = cast(List[str], d.pop("fieldDefinitionNames", UNSET))

        has_unique_residues = d.pop("hasUniqueResidues", UNSET)

        tag_schema_constraint = cls(
            field_definition_names=field_definition_names,
            has_unique_residues=has_unique_residues,
        )

        tag_schema_constraint.additional_properties = d
        return tag_schema_constraint

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.batch import Batch

T = TypeVar("T", bound="BatchesPaginatedList")


@attr.s(auto_attribs=True)
class BatchesPaginatedList:
    """  """

    batches: List[Batch]
    next_token: str

    def to_dict(self) -> Dict[str, Any]:
        batches = []
        for batches_item_data in self.batches:
            batches_item = batches_item_data.to_dict()

            batches.append(batches_item)

        next_token = self.next_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "batches": batches,
                "nextToken": next_token,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        batches = []
        _batches = d.pop("batches")
        for batches_item_data in _batches:
            batches_item = Batch.from_dict(batches_item_data)

            batches.append(batches_item)

        next_token = d.pop("nextToken")

        batches_paginated_list = cls(
            batches=batches,
            next_token=next_token,
        )

        return batches_paginated_list



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['benchling_api_client',
 'benchling_api_client.api',
 'benchling_api_client.api.aa_sequences',
 'benchling_api_client.api.assay_results',
 'benchling_api_client.api.assay_runs',
 'benchling_api_client.api.batches',
 'benchling_api_client.api.blobs',
 'benchling_api_client.api.boxes',
 'benchling_api_client.api.containers',
 'benchling_api_client.api.custom_entities',
 'benchling_api_client.api.dna_sequences',
 'benchling_api_client.api.dropdowns',
 'benchling_api_client.api.entries',
 'benchling_api_client.api.events',
 'benchling_api_client.api.exports',
 'benchling_api_client.api.folders',
 'benchling_api_client.api.inventory',
 'benchling_api_client.api.lab_automation',
 'benchling_api_client.api.label_templates',
 'benchling_api_client.api.locations',
 'benchling_api_client.api.oligos',
 'benchling_api_client.api.plates',
 'benchling_api_client.api.printers',
 'benchling_api_client.api.projects',
 'benchling_api_client.api.registry',
 'benchling_api_client.api.requests',
 'benchling_api_client.api.schemas',
 'benchling_api_client.api.tasks',
 'benchling_api_client.api.warehouse',
 'benchling_api_client.models']

package_data = \
{'': ['*']}

install_requires = \
['attrs>=20.1.0,<21.0.0',
 'httpx>=0.15.4,<0.17.0',
 'python-dateutil>=2.8.0,<3.0.0']

setup_kwargs = {
    'name': 'benchling-api-client',
    'version': '0.10.3a0',
    'description': 'A client library for accessing Benchling API',
    'long_description': '# Benchling API Client\n\nA Python 3.8+ API Client for the [Benchling](https://www.benchling.com/) platform automatically generated from OpenAPI specs.\n\n*Important!* This is an unsupported pre-release not suitable for production use.\n\n_Please reach out to your customer support representative if you would be interested in a public version!_',
    'author': 'Benchling Customer Engineering',
    'author_email': 'ce-team@benchling.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.dropdown_option_update import DropdownOptionUpdate

T = TypeVar("T", bound="DropdownUpdate")


@attr.s(auto_attribs=True)
class DropdownUpdate:
    """  """

    options: List[DropdownOptionUpdate]

    def to_dict(self) -> Dict[str, Any]:
        options = []
        for options_item_data in self.options:
            options_item = options_item_data.to_dict()

            options.append(options_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "options": options,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        options = []
        _options = d.pop("options")
        for options_item_data in _options:
            options_item = DropdownOptionUpdate.from_dict(options_item_data)

            options.append(options_item)

        dropdown_update = cls(
            options=options,
        )

        return dropdown_update



from typing import Any, Dict, Optional, Type, TypeVar

import attr

T = TypeVar("T", bound="Printer")


@attr.s(auto_attribs=True)
class Printer:
    """  """

    address: str
    id: str
    name: str
    registry_id: str
    description: Optional[str]
    port: Optional[int]

    def to_dict(self) -> Dict[str, Any]:
        address = self.address
        id = self.id
        name = self.name
        registry_id = self.registry_id
        description = self.description
        port = self.port

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "address": address,
                "id": id,
                "name": name,
                "registryId": registry_id,
                "description": description,
                "port": port,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        address = d.pop("address")

        id = d.pop("id")

        name = d.pop("name")

        registry_id = d.pop("registryId")

        description = d.pop("description")

        port = d.pop("port")

        printer = cls(
            address=address,
            id=id,
            name=name,
            registry_id=registry_id,
            description=description,
            port=port,
        )

        return printer



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EntrySchema")


@attr.s(auto_attribs=True)
class EntrySchema:
    """ Entry schema """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        entry_schema = cls(
            id=id,
            name=name,
        )

        return entry_schema



from typing import Any, Dict, Optional, Type, TypeVar

import attr

T = TypeVar("T", bound="Measurement")


@attr.s(auto_attribs=True)
class Measurement:
    """  """

    value: Optional[float]
    units: Optional[str]

    def to_dict(self) -> Dict[str, Any]:
        value = self.value
        units = self.units

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "value": value,
                "units": units,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        value = d.pop("value")

        units = d.pop("units")

        measurement = cls(
            value=value,
            units=units,
        )

        return measurement



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr
from dateutil.parser import isoparse

from ..models.custom_entity_creator import CustomEntityCreator
from ..models.custom_fields import CustomFields
from ..models.fields import Fields
from ..models.schema_summary import SchemaSummary
from ..models.user_summary import UserSummary
from ..types import UNSET, Unset

T = TypeVar("T", bound="CustomEntity")


@attr.s(auto_attribs=True)
class CustomEntity:
    """  """

    fields: Fields
    id: str
    aliases: Union[Unset, List[str]] = UNSET
    created_at: Union[Unset, datetime.datetime] = UNSET
    creator: Union[Unset, CustomEntityCreator] = UNSET
    authors: Union[Unset, List[UserSummary]] = UNSET
    custom_fields: Union[Unset, CustomFields] = UNSET
    entity_registry_id: Union[Unset, None, str] = UNSET
    folder_id: Union[Unset, str] = UNSET
    modified_at: Union[Unset, datetime.datetime] = UNSET
    name: Union[Unset, str] = UNSET
    registry_id: Union[Unset, None, str] = UNSET
    schema: Union[Unset, SchemaSummary] = UNSET
    api_url: Union[Unset, str] = UNSET
    web_url: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        fields = self.fields.to_dict()

        id = self.id
        aliases: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.aliases, Unset):
            aliases = self.aliases

        created_at: Union[Unset, str] = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        creator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.creator, Unset):
            creator = self.creator.to_dict()

        authors: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.authors, Unset):
            authors = []
            for authors_item_data in self.authors:
                authors_item = authors_item_data.to_dict()

                authors.append(authors_item)

        custom_fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_fields, Unset):
            custom_fields = self.custom_fields.to_dict()

        entity_registry_id = self.entity_registry_id
        folder_id = self.folder_id
        modified_at: Union[Unset, str] = UNSET
        if not isinstance(self.modified_at, Unset):
            modified_at = self.modified_at.isoformat()

        name = self.name
        registry_id = self.registry_id
        schema: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.schema, Unset):
            schema = self.schema.to_dict()

        api_url = self.api_url
        web_url = self.web_url

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "fields": fields,
                "id": id,
            }
        )
        if aliases is not UNSET:
            field_dict["aliases"] = aliases
        if created_at is not UNSET:
            field_dict["createdAt"] = created_at
        if creator is not UNSET:
            field_dict["creator"] = creator
        if authors is not UNSET:
            field_dict["authors"] = authors
        if custom_fields is not UNSET:
            field_dict["customFields"] = custom_fields
        if entity_registry_id is not UNSET:
            field_dict["entityRegistryId"] = entity_registry_id
        if folder_id is not UNSET:
            field_dict["folderId"] = folder_id
        if modified_at is not UNSET:
            field_dict["modifiedAt"] = modified_at
        if name is not UNSET:
            field_dict["name"] = name
        if registry_id is not UNSET:
            field_dict["registryId"] = registry_id
        if schema is not UNSET:
            field_dict["schema"] = schema
        if api_url is not UNSET:
            field_dict["apiURL"] = api_url
        if web_url is not UNSET:
            field_dict["webURL"] = web_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        fields = Fields.from_dict(d.pop("fields"))

        id = d.pop("id")

        aliases = cast(List[str], d.pop("aliases", UNSET))

        created_at = None
        _created_at = d.pop("createdAt", UNSET)
        if _created_at is not None:
            created_at = isoparse(cast(str, _created_at))

        creator: Union[Unset, CustomEntityCreator] = UNSET
        _creator = d.pop("creator", UNSET)
        if _creator is not None and not isinstance(_creator, Unset):
            creator = CustomEntityCreator.from_dict(cast(Dict[str, Any], _creator))

        authors = []
        _authors = d.pop("authors", UNSET)
        for authors_item_data in _authors or []:
            authors_item = UserSummary.from_dict(authors_item_data)

            authors.append(authors_item)

        custom_fields: Union[Unset, CustomFields] = UNSET
        _custom_fields = d.pop("customFields", UNSET)
        if _custom_fields is not None and not isinstance(_custom_fields, Unset):
            custom_fields = CustomFields.from_dict(cast(Dict[str, Any], _custom_fields))

        entity_registry_id = d.pop("entityRegistryId", UNSET)

        folder_id = d.pop("folderId", UNSET)

        modified_at = None
        _modified_at = d.pop("modifiedAt", UNSET)
        if _modified_at is not None:
            modified_at = isoparse(cast(str, _modified_at))

        name = d.pop("name", UNSET)

        registry_id = d.pop("registryId", UNSET)

        schema: Union[Unset, SchemaSummary] = UNSET
        _schema = d.pop("schema", UNSET)
        if _schema is not None and not isinstance(_schema, Unset):
            schema = SchemaSummary.from_dict(cast(Dict[str, Any], _schema))

        api_url = d.pop("apiURL", UNSET)

        web_url = d.pop("webURL", UNSET)

        custom_entity = cls(
            fields=fields,
            id=id,
            aliases=aliases,
            created_at=created_at,
            creator=creator,
            authors=authors,
            custom_fields=custom_fields,
            entity_registry_id=entity_registry_id,
            folder_id=folder_id,
            modified_at=modified_at,
            name=name,
            registry_id=registry_id,
            schema=schema,
            api_url=api_url,
            web_url=web_url,
        )

        return custom_entity



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.multiple_containers_transfer import MultipleContainersTransfer

T = TypeVar("T", bound="MultipleContainersTransfersList")


@attr.s(auto_attribs=True)
class MultipleContainersTransfersList:
    """  """

    transfers: List[MultipleContainersTransfer]

    def to_dict(self) -> Dict[str, Any]:
        transfers = []
        for transfers_item_data in self.transfers:
            transfers_item = transfers_item_data.to_dict()

            transfers.append(transfers_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "transfers": transfers,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        transfers = []
        _transfers = d.pop("transfers")
        for transfers_item_data in _transfers:
            transfers_item = MultipleContainersTransfer.from_dict(transfers_item_data)

            transfers.append(transfers_item)

        multiple_containers_transfers_list = cls(
            transfers=transfers,
        )

        return multiple_containers_transfers_list



from typing import Any, Dict, Type, TypeVar

import attr

T = TypeVar("T", bound="WarehouseCredentialsCreate")


@attr.s(auto_attribs=True)
class WarehouseCredentialsCreate:
    """  """

    expires_in: int

    def to_dict(self) -> Dict[str, Any]:
        expires_in = self.expires_in

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "expiresIn": expires_in,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        expires_in = d.pop("expiresIn")

        warehouse_credentials_create = cls(
            expires_in=expires_in,
        )

        return warehouse_credentials_create



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="BaseError")


@attr.s(auto_attribs=True)
class BaseError:
    """  """

    message: Union[Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    user_message: Union[Unset, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        message = self.message
        type = self.type
        user_message = self.user_message

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if message is not UNSET:
            field_dict["message"] = message
        if type is not UNSET:
            field_dict["type"] = type
        if user_message is not UNSET:
            field_dict["userMessage"] = user_message

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        message = d.pop("message", UNSET)

        type = d.pop("type", UNSET)

        user_message = d.pop("userMessage", UNSET)

        base_error = cls(
            message=message,
            type=type,
            user_message=user_message,
        )

        return base_error



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.blob_multipart_create_type import BlobMultipartCreateType
from ..types import UNSET, Unset

T = TypeVar("T", bound="BlobMultipartCreate")


@attr.s(auto_attribs=True)
class BlobMultipartCreate:
    """  """

    name: str
    type: BlobMultipartCreateType
    mime_type: Union[Unset, str] = "application/octet-stream"

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        type = self.type.value

        mime_type = self.mime_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "type": type,
            }
        )
        if mime_type is not UNSET:
            field_dict["mimeType"] = mime_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        type = BlobMultipartCreateType(d.pop("type"))

        mime_type = d.pop("mimeType", UNSET)

        blob_multipart_create = cls(
            name=name,
            type=type,
            mime_type=mime_type,
        )

        return blob_multipart_create



from typing import Any, Dict, Type, TypeVar, Union, cast

import attr

from ..models.custom_fields import CustomFields
from ..models.fields import Fields
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryCreate")


@attr.s(auto_attribs=True)
class EntryCreate:
    """  """

    name: str
    folder_id: str
    author_ids: Union[Unset, str] = UNSET
    entry_template_id: Union[Unset, str] = UNSET
    schema_id: Union[Unset, str] = UNSET
    custom_fields: Union[Unset, CustomFields] = UNSET
    fields: Union[Unset, Fields] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        folder_id = self.folder_id
        author_ids = self.author_ids
        entry_template_id = self.entry_template_id
        schema_id = self.schema_id
        custom_fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_fields, Unset):
            custom_fields = self.custom_fields.to_dict()

        fields: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.fields, Unset):
            fields = self.fields.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "name": name,
                "folderId": folder_id,
            }
        )
        if author_ids is not UNSET:
            field_dict["authorIds"] = author_ids
        if entry_template_id is not UNSET:
            field_dict["entryTemplateId"] = entry_template_id
        if schema_id is not UNSET:
            field_dict["schemaId"] = schema_id
        if custom_fields is not UNSET:
            field_dict["customFields"] = custom_fields
        if fields is not UNSET:
            field_dict["fields"] = fields

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name")

        folder_id = d.pop("folderId")

        author_ids = d.pop("authorIds", UNSET)

        entry_template_id = d.pop("entryTemplateId", UNSET)

        schema_id = d.pop("schemaId", UNSET)

        custom_fields: Union[Unset, CustomFields] = UNSET
        _custom_fields = d.pop("customFields", UNSET)
        if _custom_fields is not None and not isinstance(_custom_fields, Unset):
            custom_fields = CustomFields.from_dict(cast(Dict[str, Any], _custom_fields))

        fields: Union[Unset, Fields] = UNSET
        _fields = d.pop("fields", UNSET)
        if _fields is not None and not isinstance(_fields, Unset):
            fields = Fields.from_dict(cast(Dict[str, Any], _fields))

        entry_create = cls(
            name=name,
            folder_id=folder_id,
            author_ids=author_ids,
            entry_template_id=entry_template_id,
            schema_id=schema_id,
            custom_fields=custom_fields,
            fields=fields,
        )

        return entry_create



from typing import Any, Dict, Type, TypeVar, Union, cast

import attr

from ..models.organization import Organization
from ..types import UNSET, Unset

T = TypeVar("T", bound="Registry")


@attr.s(auto_attribs=True)
class Registry:
    """  """

    id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    owner: Union[Unset, Organization] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        name = self.name
        owner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner, Unset):
            owner = self.owner.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if name is not UNSET:
            field_dict["name"] = name
        if owner is not UNSET:
            field_dict["owner"] = owner

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        name = d.pop("name", UNSET)

        owner: Union[Unset, Organization] = UNSET
        _owner = d.pop("owner", UNSET)
        if _owner is not None and not isinstance(_owner, Unset):
            owner = Organization.from_dict(cast(Dict[str, Any], _owner))

        registry = cls(
            id=id,
            name=name,
            owner=owner,
        )

        return registry



from typing import Any, Dict, Type, TypeVar

import attr

from ..models.measurement import Measurement

T = TypeVar("T", bound="ContainerContentUpdate")


@attr.s(auto_attribs=True)
class ContainerContentUpdate:
    """  """

    concentration: Measurement

    def to_dict(self) -> Dict[str, Any]:
        concentration = self.concentration.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "concentration": concentration,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        concentration = Measurement.from_dict(d.pop("concentration"))

        container_content_update = cls(
            concentration=concentration,
        )

        return container_content_update



from enum import Enum


class ListEntriesReviewStatus(str, Enum):
    ACCEPTED = "ACCEPTED"
    REJECTED = "REJECTED"
    NEEDS_REVIEW = "NEEDS_REVIEW"
    RETRACTED = "RETRACTED"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class NotePartType(str, Enum):
    TEXT = "text"
    CODE = "code"
    TABLE = "table"
    LIST_BULLET = "list_bullet"
    LIST_NUMBER = "list_number"
    LIST_CHECKBOX = "list_checkbox"
    EXTERNAL_FILE = "external_file"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="EntriesArchivalChange")


@attr.s(auto_attribs=True)
class EntriesArchivalChange:
    """IDs of all items that were archived or unarchived, grouped by resource type. This includes the IDs of entries that changed.."""

    entry_ids: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        entry_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.entry_ids, Unset):
            entry_ids = self.entry_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if entry_ids is not UNSET:
            field_dict["entryIds"] = entry_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        entry_ids = cast(List[str], d.pop("entryIds", UNSET))

        entries_archival_change = cls(
            entry_ids=entry_ids,
        )

        return entries_archival_change



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AaSequencesArchivalChange")


@attr.s(auto_attribs=True)
class AaSequencesArchivalChange:
    """IDs of all items that were archived or unarchived, grouped by resource type. This includes the IDs of AA sequences along with any IDs of batches that were archived / unarchived."""

    aa_sequence_ids: Union[Unset, List[str]] = UNSET
    batch_ids: Union[Unset, List[str]] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        aa_sequence_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.aa_sequence_ids, Unset):
            aa_sequence_ids = self.aa_sequence_ids

        batch_ids: Union[Unset, List[Any]] = UNSET
        if not isinstance(self.batch_ids, Unset):
            batch_ids = self.batch_ids

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if aa_sequence_ids is not UNSET:
            field_dict["aaSequenceIds"] = aa_sequence_ids
        if batch_ids is not UNSET:
            field_dict["batchIds"] = batch_ids

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        aa_sequence_ids = cast(List[str], d.pop("aaSequenceIds", UNSET))

        batch_ids = cast(List[str], d.pop("batchIds", UNSET))

        aa_sequences_archival_change = cls(
            aa_sequence_ids=aa_sequence_ids,
            batch_ids=batch_ids,
        )

        return aa_sequences_archival_change



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.request_response_samples_item_batch import RequestResponseSamplesItemBatch
from ..models.request_response_samples_item_entity import RequestResponseSamplesItemEntity
from ..models.request_response_samples_item_status import RequestResponseSamplesItemStatus
from ..types import UNSET, Unset

T = TypeVar("T", bound="RequestResponseSamplesItem")


@attr.s(auto_attribs=True)
class RequestResponseSamplesItem:
    """  """

    status: Union[Unset, RequestResponseSamplesItemStatus] = UNSET
    entity: Union[Unset, RequestResponseSamplesItemEntity] = UNSET
    batch: Union[Unset, RequestResponseSamplesItemBatch] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        status: Union[Unset, RequestResponseSamplesItemStatus] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status

        entity: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.entity, Unset):
            entity = self.entity.to_dict()

        batch: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.batch, Unset):
            batch = self.batch.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if status is not UNSET:
            field_dict["status"] = status
        if entity is not UNSET:
            field_dict["entity"] = entity
        if batch is not UNSET:
            field_dict["batch"] = batch

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        status = None
        _status = d.pop("status", UNSET)
        if _status is not None and _status is not UNSET:
            status = RequestResponseSamplesItemStatus(_status)

        entity: Union[Unset, RequestResponseSamplesItemEntity] = UNSET
        _entity = d.pop("entity", UNSET)
        if _entity is not None and not isinstance(_entity, Unset):
            entity = RequestResponseSamplesItemEntity.from_dict(cast(Dict[str, Any], _entity))

        batch: Union[Unset, RequestResponseSamplesItemBatch] = UNSET
        _batch = d.pop("batch", UNSET)
        if _batch is not None and not isinstance(_batch, Unset):
            batch = RequestResponseSamplesItemBatch.from_dict(cast(Dict[str, Any], _batch))

        request_response_samples_item = cls(
            status=status,
            entity=entity,
            batch=batch,
        )

        request_response_samples_item.additional_properties = d
        return request_response_samples_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Optional, Union

import httpx

from ...client import Client
from ...models.bad_request_error import BadRequestError
from ...models.custom_entities_paginated_list import CustomEntitiesPaginatedList
from ...models.list_custom_entities_sort import ListCustomEntitiesSort
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    client: Client,
    next_token: Union[Unset, None, str] = UNSET,
    page_size: Union[Unset, None, int] = 50,
    sort: Union[Unset, None, ListCustomEntitiesSort] = ListCustomEntitiesSort.MODIFIEDATDESC,
    modified_at: Union[Unset, None, str] = UNSET,
    name: Union[Unset, None, str] = UNSET,
    name_includes: Union[Unset, None, str] = UNSET,
    folder_id: Union[Unset, None, str] = UNSET,
    mentioned_in: Union[Unset, None, List[str]] = UNSET,
    project_id: Union[Unset, None, str] = UNSET,
    registry_id: Union[Unset, None, str] = UNSET,
    schema_id: Union[Unset, None, str] = UNSET,
    archive_reason: Union[Unset, None, str] = UNSET,
    mentions: Union[Unset, None, List[str]] = UNSET,
    ids: Union[Unset, None, str] = UNSET,
    entity_registry_idsany_of: Union[Unset, None, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/custom-entities".format(client.base_url)

    headers: Dict[str, Any] = client.get_headers()

    json_sort: Union[Unset, ListCustomEntitiesSort] = UNSET
    if not isinstance(sort, Unset) and sort is not None:
        json_sort = sort

    json_mentioned_in: Union[Unset, List[Any]] = UNSET
    if not isinstance(mentioned_in, Unset) and mentioned_in is not None:
        json_mentioned_in = mentioned_in

    json_mentions: Union[Unset, List[Any]] = UNSET
    if not isinstance(mentions, Unset) and mentions is not None:
        json_mentions = mentions

    params: Dict[str, Any] = {}
    if next_token is not UNSET and next_token is not None:
        params["nextToken"] = next_token
    if page_size is not UNSET and page_size is not None:
        params["pageSize"] = page_size
    if sort is not UNSET and sort is not None:
        params["sort"] = json_sort
    if modified_at is not UNSET and modified_at is not None:
        params["modifiedAt"] = modified_at
    if name is not UNSET and name is not None:
        params["name"] = name
    if name_includes is not UNSET and name_includes is not None:
        params["nameIncludes"] = name_includes
    if folder_id is not UNSET and folder_id is not None:
        params["folderId"] = folder_id
    if mentioned_in is not UNSET and mentioned_in is not None:
        params["mentionedIn"] = json_mentioned_in
    if project_id is not UNSET and project_id is not None:
        params["projectId"] = project_id
    if registry_id is not UNSET and registry_id is not None:
        params["registryId"] = registry_id
    if schema_id is not UNSET and schema_id is not None:
        params["schemaId"] = schema_id
    if archive_reason is not UNSET and archive_reason is not None:
        params["archiveReason"] = archive_reason
    if mentions is not UNSET and mentions is not None:
        params["mentions"] = json_mentions
    if ids is not UNSET and ids is not None:
        params["ids"] = ids
    if entity_registry_idsany_of is not UNSET and entity_registry_idsany_of is not None:
        params["entityRegistryIds.anyOf"] = entity_registry_idsany_of

    return {
        "url": url,
        "headers": headers,
        "cookies": client.get_cookies(),
        "timeout": client.get_timeout(),
        "params": params,
    }


def _parse_response(*, response: httpx.Response) -> Optional[Union[CustomEntitiesPaginatedList, BadRequestError]]:
    if response.status_code == 200:
        response_200 = CustomEntitiesPaginatedList.from_dict(response.json())

        return response_200
    if response.status_code == 400:
        response_400 = BadRequestError.from_dict(response.json())

        return response_400
    return None


def _build_response(*, response: httpx.Response) -> Response[Union[CustomEntitiesPaginatedList, BadRequestError]]:
    return Response(
        status_code=response.status_code,
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(response=response),
    )


def sync_detailed(
    *,
    client: Client,
    next_token: Union[Unset, None, str] = UNSET,
    page_size: Union[Unset, None, int] = 50,
    sort: Union[Unset, None, ListCustomEntitiesSort] = ListCustomEntitiesSort.MODIFIEDATDESC,
    modified_at: Union[Unset, None, str] = UNSET,
    name: Union[Unset, None, str] = UNSET,
    name_includes: Union[Unset, None, str] = UNSET,
    folder_id: Union[Unset, None, str] = UNSET,
    mentioned_in: Union[Unset, None, List[str]] = UNSET,
    project_id: Union[Unset, None, str] = UNSET,
    registry_id: Union[Unset, None, str] = UNSET,
    schema_id: Union[Unset, None, str] = UNSET,
    archive_reason: Union[Unset, None, str] = UNSET,
    mentions: Union[Unset, None, List[str]] = UNSET,
    ids: Union[Unset, None, str] = UNSET,
    entity_registry_idsany_of: Union[Unset, None, str] = UNSET,
) -> Response[Union[CustomEntitiesPaginatedList, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        next_token=next_token,
        page_size=page_size,
        sort=sort,
        modified_at=modified_at,
        name=name,
        name_includes=name_includes,
        folder_id=folder_id,
        mentioned_in=mentioned_in,
        project_id=project_id,
        registry_id=registry_id,
        schema_id=schema_id,
        archive_reason=archive_reason,
        mentions=mentions,
        ids=ids,
        entity_registry_idsany_of=entity_registry_idsany_of,
    )

    response = httpx.get(
        **kwargs,
    )

    return _build_response(response=response)


def sync(
    *,
    client: Client,
    next_token: Union[Unset, None, str] = UNSET,
    page_size: Union[Unset, None, int] = 50,
    sort: Union[Unset, None, ListCustomEntitiesSort] = ListCustomEntitiesSort.MODIFIEDATDESC,
    modified_at: Union[Unset, None, str] = UNSET,
    name: Union[Unset, None, str] = UNSET,
    name_includes: Union[Unset, None, str] = UNSET,
    folder_id: Union[Unset, None, str] = UNSET,
    mentioned_in: Union[Unset, None, List[str]] = UNSET,
    project_id: Union[Unset, None, str] = UNSET,
    registry_id: Union[Unset, None, str] = UNSET,
    schema_id: Union[Unset, None, str] = UNSET,
    archive_reason: Union[Unset, None, str] = UNSET,
    mentions: Union[Unset, None, List[str]] = UNSET,
    ids: Union[Unset, None, str] = UNSET,
    entity_registry_idsany_of: Union[Unset, None, str] = UNSET,
) -> Optional[Union[CustomEntitiesPaginatedList, BadRequestError]]:
    """ List custom entities """

    return sync_detailed(
        client=client,
        next_token=next_token,
        page_size=page_size,
        sort=sort,
        modified_at=modified_at,
        name=name,
        name_includes=name_includes,
        folder_id=folder_id,
        mentioned_in=mentioned_in,
        project_id=project_id,
        registry_id=registry_id,
        schema_id=schema_id,
        archive_reason=archive_reason,
        mentions=mentions,
        ids=ids,
        entity_registry_idsany_of=entity_registry_idsany_of,
    ).parsed


async def asyncio_detailed(
    *,
    client: Client,
    next_token: Union[Unset, None, str] = UNSET,
    page_size: Union[Unset, None, int] = 50,
    sort: Union[Unset, None, ListCustomEntitiesSort] = ListCustomEntitiesSort.MODIFIEDATDESC,
    modified_at: Union[Unset, None, str] = UNSET,
    name: Union[Unset, None, str] = UNSET,
    name_includes: Union[Unset, None, str] = UNSET,
    folder_id: Union[Unset, None, str] = UNSET,
    mentioned_in: Union[Unset, None, List[str]] = UNSET,
    project_id: Union[Unset, None, str] = UNSET,
    registry_id: Union[Unset, None, str] = UNSET,
    schema_id: Union[Unset, None, str] = UNSET,
    archive_reason: Union[Unset, None, str] = UNSET,
    mentions: Union[Unset, None, List[str]] = UNSET,
    ids: Union[Unset, None, str] = UNSET,
    entity_registry_idsany_of: Union[Unset, None, str] = UNSET,
) -> Response[Union[CustomEntitiesPaginatedList, BadRequestError]]:
    kwargs = _get_kwargs(
        client=client,
        next_token=next_token,
        page_size=page_size,
        sort=sort,
        modified_at=modified_at,
        name=name,
        name_includes=name_includes,
        folder_id=folder_id,
        mentioned_in=mentioned_in,
        project_id=project_id,
        registry_id=registry_id,
        schema_id=schema_id,
        archive_reason=archive_reason,
        mentions=mentions,
        ids=ids,
        entity_registry_idsany_of=entity_registry_idsany_of,
    )

    async with httpx.AsyncClient() as _client:
        response = await _client.get(**kwargs)

    return _build_response(response=response)


async def asyncio(
    *,
    client: Client,
    next_token: Union[Unset, None, str] = UNSET,
    page_size: Union[Unset, None, int] = 50,
    sort: Union[Unset, None, ListCustomEntitiesSort] = ListCustomEntitiesSort.MODIFIEDATDESC,
    modified_at: Union[Unset, None, str] = UNSET,
    name: Union[Unset, None, str] = UNSET,
    name_includes: Union[Unset, None, str] = UNSET,
    folder_id: Union[Unset, None, str] = UNSET,
    mentioned_in: Union[Unset, None, List[str]] = UNSET,
    project_id: Union[Unset, None, str] = UNSET,
    registry_id: Union[Unset, None, str] = UNSET,
    schema_id: Union[Unset, None, str] = UNSET,
    archive_reason: Union[Unset, None, str] = UNSET,
    mentions: Union[Unset, None, List[str]] = UNSET,
    ids: Union[Unset, None, str] = UNSET,
    entity_registry_idsany_of: Union[Unset, None, str] = UNSET,
) -> Optional[Union[CustomEntitiesPaginatedList, BadRequestError]]:
    """ List custom entities """

    return (
        await asyncio_detailed(
            client=client,
            next_token=next_token,
            page_size=page_size,
            sort=sort,
            modified_at=modified_at,
            name=name,
            name_includes=name_includes,
            folder_id=folder_id,
            mentioned_in=mentioned_in,
            project_id=project_id,
            registry_id=registry_id,
            schema_id=schema_id,
            archive_reason=archive_reason,
            mentions=mentions,
            ids=ids,
            entity_registry_idsany_of=entity_registry_idsany_of,
        )
    ).parsed



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="Primer")


@attr.s(auto_attribs=True)
class Primer:
    """  """

    bases: Union[Unset, str] = UNSET
    bind_position: Union[Unset, int] = UNSET
    color: Union[Unset, str] = UNSET
    start: Union[Unset, int] = UNSET
    end: Union[Unset, int] = UNSET
    name: Union[Unset, str] = UNSET
    oligo_id: Union[Unset, str] = UNSET
    overhang_length: Union[Unset, int] = UNSET
    strand: Union[Unset, int] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        bases = self.bases
        bind_position = self.bind_position
        color = self.color
        start = self.start
        end = self.end
        name = self.name
        oligo_id = self.oligo_id
        overhang_length = self.overhang_length
        strand = self.strand

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if bases is not UNSET:
            field_dict["bases"] = bases
        if bind_position is not UNSET:
            field_dict["bindPosition"] = bind_position
        if color is not UNSET:
            field_dict["color"] = color
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end
        if name is not UNSET:
            field_dict["name"] = name
        if oligo_id is not UNSET:
            field_dict["oligoId"] = oligo_id
        if overhang_length is not UNSET:
            field_dict["overhangLength"] = overhang_length
        if strand is not UNSET:
            field_dict["strand"] = strand

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        bases = d.pop("bases", UNSET)

        bind_position = d.pop("bindPosition", UNSET)

        color = d.pop("color", UNSET)

        start = d.pop("start", UNSET)

        end = d.pop("end", UNSET)

        name = d.pop("name", UNSET)

        oligo_id = d.pop("oligoId", UNSET)

        overhang_length = d.pop("overhangLength", UNSET)

        strand = d.pop("strand", UNSET)

        primer = cls(
            bases=bases,
            bind_position=bind_position,
            color=color,
            start=start,
            end=end,
            name=name,
            oligo_id=oligo_id,
            overhang_length=overhang_length,
            strand=strand,
        )

        return primer



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.dropdown_summary import DropdownSummary

T = TypeVar("T", bound="DropdownSummariesPaginatedList")


@attr.s(auto_attribs=True)
class DropdownSummariesPaginatedList:
    """  """

    dropdowns: List[DropdownSummary]
    next_token: str

    def to_dict(self) -> Dict[str, Any]:
        dropdowns = []
        for dropdowns_item_data in self.dropdowns:
            dropdowns_item = dropdowns_item_data.to_dict()

            dropdowns.append(dropdowns_item)

        next_token = self.next_token

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "dropdowns": dropdowns,
                "nextToken": next_token,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        dropdowns = []
        _dropdowns = d.pop("dropdowns")
        for dropdowns_item_data in _dropdowns:
            dropdowns_item = DropdownSummary.from_dict(dropdowns_item_data)

            dropdowns.append(dropdowns_item)

        next_token = d.pop("nextToken")

        dropdown_summaries_paginated_list = cls(
            dropdowns=dropdowns,
            next_token=next_token,
        )

        return dropdown_summaries_paginated_list



from typing import Any, Dict, Type, TypeVar, Union

import attr

from ..models.entry_link_type import EntryLinkType
from ..types import UNSET, Unset

T = TypeVar("T", bound="EntryLink")


@attr.s(auto_attribs=True)
class EntryLink:
    """Links are contained within notes to reference resources that live outside of the entry. A link can target an external resource via an http(s):// hyperlink or a Benchling resource via @-mentions and drag-n-drop."""

    id: Union[Unset, str] = UNSET
    type: Union[Unset, EntryLinkType] = UNSET
    web_url: Union[Unset, None, str] = UNSET

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        type: Union[Unset, EntryLinkType] = UNSET
        if not isinstance(self.type, Unset):
            type = self.type

        web_url = self.web_url

        field_dict: Dict[str, Any] = {}
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if type is not UNSET:
            field_dict["type"] = type
        if web_url is not UNSET:
            field_dict["webURL"] = web_url

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET)

        type = None
        _type = d.pop("type", UNSET)
        if _type is not None and _type is not UNSET:
            type = EntryLinkType(_type)

        web_url = d.pop("webURL", UNSET)

        entry_link = cls(
            id=id,
            type=type,
            web_url=web_url,
        )

        return entry_link



import datetime
from typing import Any, Dict, Type, TypeVar

import attr
from dateutil.parser import isoparse

T = TypeVar("T", bound="WarehouseCredentials")


@attr.s(auto_attribs=True)
class WarehouseCredentials:
    """  """

    expires_at: datetime.datetime
    username: str
    password: str

    def to_dict(self) -> Dict[str, Any]:
        expires_at = self.expires_at.isoformat()

        username = self.username
        password = self.password

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {
                "expiresAt": expires_at,
                "username": username,
                "password": password,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        expires_at = isoparse(d.pop("expiresAt"))

        username = d.pop("username")

        password = d.pop("password")

        warehouse_credentials = cls(
            expires_at=expires_at,
            username=username,
            password=password,
        )

        return warehouse_credentials
